
import java.applet.Applet;
import java.awt.Graphics;
import java.awt.Color;

public class RotatingPoints extends Applet
	implements Runnable
	{
	int numPoints;

	double[] x, y, z;    //the source points
	double[] xt, yt, zt; //the transformed points
	Color[] color;       //their colors

	//their angles in each dimension
	double xAngle, yAngle, zAngle;

	//their rotation speed
	double angularVelocity;

	Thread thread;

	int windowWidth, windowHeight;
	int xOrigin, yOrigin;

	///////////////////////////////////////////////////////////////////////
	public void init()
		{
		//set the origin
		windowWidth = this.getSize().width;
		windowHeight = this.getSize().height;
		xOrigin = windowWidth / 2;
		yOrigin = windowHeight / 2;

		//create some random colored points
		numPoints = 5;

		x = new double[numPoints];
		y = new double[numPoints];
		z = new double[numPoints];

		xt = new double[numPoints];
		yt = new double[numPoints];
		zt = new double[numPoints];

		color = new Color[numPoints];

		for (int i = 0; i < numPoints; i++)
			{
			x[i] = windowWidth * (Math.random() - 0.5);
			y[i] = windowHeight * (Math.random() - 0.5);
			z[i] = 100 * (Math.random() - 0.5);

			color[i] = new Color((i * 8) % 255, (i * 16) % 255, (i * 32) % 255);
			}

		//set the angular velocity
		angularVelocity = Math.PI / 100;

		//and start the thread
		thread = new Thread(this);
		thread.start();
		}

	///////////////////////////////////////////////////////////////////////
	public void run()
		{
		while (thread != null)
			{
			rotatePoints(xAngle, yAngle, zAngle);

			xAngle += angularVelocity;
			yAngle += angularVelocity / 2;
			zAngle += angularVelocity / 4;

			repaint();

			try {thread.sleep(100);}
			catch (InterruptedException ignored) {}
			}
		}

	///////////////////////////////////////////////////////////////////////
	public void start()
		{
		if (thread == null)
			{
			thread = new Thread(this);
			thread.start();
			}
		}

	///////////////////////////////////////////////////////////////////////
	public void stop()
		{
		if (thread != null)
			{
			thread.stop();
			thread = null;
			}
		}

	///////////////////////////////////////////////////////////////////////
	public void paint(Graphics graphicsContext)
		{
		for (int i = 0; i < numPoints; i++)
			{
			int x = (int) xt[i];
			int y = (int) yt[i];
			graphicsContext.setColor(color[i]);
			graphicsContext.drawLine(xOrigin, yOrigin, xOrigin + x, yOrigin + y);
			graphicsContext.fillOval(xOrigin + x - 4, yOrigin + y - 4, 8, 8);
			}
		}

	///////////////////////////////////////////////////////////////////////
	private void rotatePoints(double xAngle, double yAngle, double zAngle)
		{
		double Xa, Ya, Za, Xb, Yb, Zb;
		for (int i = 0; i < numPoints; i++)
			{
			//rotate about the z-axis
			Xa = x[i] * Math.cos(xAngle) - y[i] * Math.sin(xAngle);
			Ya = x[i] * Math.sin(xAngle) + y[i] * Math.cos(xAngle);
			Za = z[i];

			//rotate about the x-axis
			Xb = Xa;
			Yb = Ya * Math.cos(yAngle) - Za * Math.sin(yAngle);
			Zb = Ya * Math.sin(yAngle) + Za * Math.cos(yAngle);

			//rotate about the y-axis
			xt[i] = Xb * Math.cos(zAngle) + Zb * Math.sin(zAngle);
			yt[i] = Yb;
			zt[i] = -Xb * Math.sin(zAngle) + Zb * Math.cos(zAngle);
			}
		}
	}
